program AVR_EnvSensor;

{ $BOOTRST $0F000}         {Reset Jump to $0F000}
{$NOSHADOW}
{ $W+ Warnings}            {Warnings off}

Device = mega128, VCC = 5;

Define_Fuses
  Override_Fuses;
  COMport        = USB;
  LockBits0      = [];
  FuseBits0      = [BODEN];
  FuseBits1      = [CKOPT, SPIEN, JTAGEN, OCDEN];
  FuseBits2      = [];
  ProgMode       = JTAG;

Import SysTick, SerPort, ADCPort, LCDmultiPort, TWImaster, SHT11drvr;

From System Import LongWord, LongInt, Float;

Define
  ProcClock      = 16000000;        {Hertz}
  SysTick        = 10;             {msec}
  StackSize      = $0064, iData;
  FrameSize      = $0064, iData;

  SerPort        = 19200, Stop2;    {Baud, StopBits|Parity}
  RxBuffer       = 16, iData;
  TxBuffer       = 16, iData;

//  SHT11drvr      = polled, 10;          // polling only
  SHT11drvr      = SysTickChecked;
  SHT11clk       = PortD, 6;       {Port, Pin#}
  SHT11Dat       = PortD, 7;       {Port, Pin#}

  ADCchans       = 1, iData;
  ADCpresc       = 128;

  TWIpresc       = TWI_BR400;
  LCDmultiPort   = I2C_TWI;
  LCDrows_M      = 4;              {rows}
  LCDcolumns_M   = 20;             {columns per line}
  LCDtype_M      = 44780;

Implementation

{$IDATA}

{--------------------------------------------------------------}
{ Type Declarations }

type


{--------------------------------------------------------------}
{ Const Declarations }
const
  deg            : char      = #$DF;


{--------------------------------------------------------------}
{ Var Declarations }
{$IDATA}
var
  bb             : byte;
  ww             : word;
  ii             : integer;
  st             : string[16];
  SHTok          : boolean;


{--------------------------------------------------------------}
{ functions }



{--------------------------------------------------------------}
{ Main Program }
{$IDATA}

begin
  LCDsetup_M(LCD_m1);
  LCDcursor_M(LCD_m1, false, false);
  Write(LCDOut_M,  'E-LAB WeatherStation');

  SHT11synchronize;
  if SHT11getStatus = $ff then                // SHT11 not present or defect
    LCDxy_M(LCD_m1, 0, 2);
    Write(LCDOut_M,  'SHT Check failed');
  else
    SHTok:= true;
  endif;

  SHT11softReset;

//  SHT11setStatus(1);                        // switch to 8/12bit resolution

  EnableInts;

  loop
    ww:= GetADC;
    LCDxy_M(LCD_m1, 0, 1);
    if ww = 0 then
      bb:= SHT11getStatus;
      st:= ByteToHex(bb);
      Write(LCDOut_M,  'Status   : $' + st);
    else
      ww:= MulDivInt(ww, 1200, 1023);
      Write(LCDOut_M,  'Pressure : ' + IntToStr(ww:4)+'hPa');
    endif;
    mDelay(100);

    if SHTok then
      LCDxy_M(LCD_m1, 0, 2);
      SHT11startTemp;
      repeat
      until SHT11ConvState;
      ww:= SHT11getTemp;
      ii:= integer(ww div 10) - 400;               // 14bit resolution
      st:= IntToStr(ii : 5 : 1);
      Write(LCDOut_M,  'Temperat :' + st + deg);
      mDelay(100);

      LCDxy_M(LCD_m1, 0, 3);
      SHT11startHum;
      repeat
      until SHT11ConvState;
      ww:= SHT11getHum;
      ww:= MulDivInt(ww, 405, 1000) - 40;         // 12bit resolution
      st:= IntToStr(ww : 4 : 1);
      Write(LCDOut_M,  'Humidity : ' + st + '%');
    endif;
    mDelay(1000);
  endloop;
end AVR_EnvSensor.

